import time
from datetime import timedelta

from httpx import AsyncClient
from nonebot.plugin import on_fullmatch, on_command
from nonebot.adapters.onebot.v11.message import Message
from nonebot.adapters.onebot.v11 import Bot, PrivateMessageEvent, GroupMessageEvent
from nonebot.params import CommandArg
from nonebot.rule import to_me
from typing import List
from .aiosqlite import *
from .config import ContestType
from .utils import *
import asyncio

###列表下标0为比赛名称、下标1为比赛时间、下标2为比赛链接
cf: List[ContestType] = []


async def get_today_data() -> List[ContestType]:
    """

    :return: 返回cf今天的比赛列表
    """
    global cf
    today = datetime.datetime.now().date()
    contest_list: List[ContestType] = []
    if len(cf) == 0:
        await get_data_cf()
    if len(cf) > 0:
        for each in cf:
            cur_time = datetime.datetime.strptime(str(each.get_time()), "%Y-%m-%d %H:%M").date()
            if cur_time == today:
                contest_list.append(each)

    return contest_list


async def get_next_data() -> List[ContestType]:
    """

    :return: 返回cf接下来的比赛列表
    """
    global cf
    tomorrow = datetime.datetime.now().date() + timedelta(days=1)
    contest_list: List[ContestType] = []
    if len(cf) == 0:
        await get_data_cf()
    if len(cf) > 0:
        for each in cf:
            cur_time = datetime.datetime.strptime(str(each.get_time()), "%Y-%m-%d %H:%M").date()
            if cur_time >= tomorrow:
                contest_list.append(each)

    return contest_list


async def get_data_cf() -> bool:
    global cf
    url = 'https://codeforces.com/api/contest.list?gym=false'
    num = 0  # 尝试获取的次数，最多尝试三次
    while num < 3:
        try:
            if len(cf) > 0:
                cf.clear()
            async with AsyncClient() as client:
                r = await client.get(url, timeout=10)
            for each in r.json()['result'][0::]:
                if each['phase'] != "BEFORE":
                    break
                contest_name = each['name']
                contest_time = int(each['startTimeSeconds'])
                contest_url = f"https://codeforces.com/contest/{str(each['id'])}"
                contest_length = int(each['durationSeconds'])

                cf.append(ContestType(contest_name, contest_time, contest_url, contest_length))
            cf.reverse()
            return True
        except:
            num += 1
            await asyncio.sleep(2)  # 两秒后再次获取信息
    return False


async def ans_cf() -> str:
    global cf
    if len(cf) == 0:
        await get_data_cf()
    if len(cf) == 0:
        return f'突然出错了，稍后再试哦~'
    msg = []
    tot = 0
    for each in cf:
        msg.append(to_context(each))
        tot += 1
        if tot >= 3:
            break
    return '\n'.join( [f"找到最近的 {tot} 场cf比赛为："] + msg )


cf_matcher = on_fullmatch('cf', priority=80, block=True)
bind = on_command('cf监视', rule=to_me(), priority=80, block=True)
bind_remove = on_command('cf监视移除', rule=to_me(), priority=80, block=True)
bind_list = on_fullmatch('cf监视列表', rule=to_me(), priority=78, block=True)
query = on_command('cf查询', rule=to_me(), priority=79, block=True)
rank = on_command('cf排名', rule=to_me(), priority=80, block=True)
remarks = on_command('cf监视备注', rule=to_me(), priority=80, block=True)


@cf_matcher.handle()
async def reply_handle():
    msg = await ans_cf()
    await cf_matcher.finish(msg)


@bind.handle()
async def bind_handle(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
    cfid = args.extract_plain_text()
    if cfid:
        status = await addUser(cfid)
        if status:
            await bind.finish(f'监视{cfid}成功！')
        else:
            await bind.finish(f'监视{cfid}失败！请检查该用户是否存在')
    else:
        await bind.finish('绑定失败，请按照格式发起指令！')


@bind_remove.handle()
async def bind_remove_handle(args: Message = CommandArg()):
    cfid = args.extract_plain_text()
    if cfid:
        status = await removeUser(cfid)
        if status:
            await bind_remove.finish(f'{cfid}移除成功')
        else:
            await bind_remove.finish(f'{cfid}移除失败')


@bind_list.handle()
async def bind_list_handle():
    msg = await returnBindList()
    await bind_list.finish(msg)


@query.handle()
async def query_handle(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
    cfid = args.extract_plain_text()
    if cfid:
        msg = await queryUser(cfid)
        await query.finish(msg)
    else:
        await bind.finish('绑定失败，请按照格式发起指令！')


@rank.handle()
async def rank_handle(bot: Bot):
    data = await returnRanklist()
    msg = '当前rating排名如下：\n'
    for person in data:
        msg += f'{person[0]} {person[1]}\n'

    await rank.finish(msg)


@remarks.handle()
async def remarks_handle(bot: Bot, args: Message = CommandArg()):
    cf_list = args.extract_plain_text().split(' ')
    if len(cf_list) != 2:
        remarks.finish('格式不正确，请重新触发指令')

    cf_id, cf_remarks = cf_list
    status = await modifyRemarks(cf_id, cf_remarks)
    if status:
        await remarks.finish('设置成功！')
    else:
        await remarks.finish('设置失败！')



from nonebot.adapters.onebot.v11 import Event

from .config import *
from .codeforces import *
from .nowcoder import *
from .atcoder import *
from .aiosqlite import *
from .utils import *
import datetime
from datetime import timedelta
from nonebot.plugin import on_fullmatch
from nonebot import require, get_bot
from nonebot.rule import to_me
from nonebot.log import logger
import asyncio

cp_broadcast_list = cp_broadcast_config.cp_broadcast_list
cp_broadcast_botname = cp_broadcast_config.cp_broadcast_botname
cp_broadcast_time = cp_broadcast_config.cp_broadcast_time
cp_broadcast_updatetime = cp_broadcast_config.cp_broadcast_updatetime
cp_broadcast_cf_list = cp_broadcast_config.cp_broadcast_cf_list
cp_broadcast_cf_interval = cp_broadcast_config.cp_broadcast_cf_interval

from nonebot.plugin import PluginMetadata

__plugin_meta__ = PluginMetadata(
    name="算法竞赛比赛查询",
    description="可以查询牛客、atcoder、codeforces平台的今天和近几天的比赛信息",
    usage= \
        "cf->查询cf比赛\n" \
        "@{botname} cf查询+id->查询某人信息\n" \
        "@{botname} cf监视+id->监视某人rating变化\n" \
        "@{botname} cf监视移除+id->不再监视某人rating变化\n" \
        "@{botname} cf监视列表->展示已监视的选手id\n" \
        "@{botname} cf排名->展示已监视选手的排名\n" \
        "@{botname} cf监视备注->给某个监视选手备注\n" \
        "nc/牛客->查询牛客比赛\n" \
        "atc->查询atcoder比赛\n" \
        "today->查询今天的比赛\n" \
        "next->查询明天之后的部分比赛\n"
        "update->更新比赛数据\n" \
        "about->{botname}的一些信息\n",
    type="application",
    homepage="https://github.com/HuParry/nonebot-plugin-cp-broadcast",
    config=Config,
    supported_adapters={"nonebot.adapters.onebot.v11"},
)

try:
    scheduler = require("nonebot_plugin_apscheduler").scheduler
except BaseException:
    scheduler = None

logger.opt(colors=True).info(
    "已检测到软依赖<y>nonebot_plugin_apscheduler</y>, <g>开启定时任务功能</g>"
    if scheduler
    else "未检测到软依赖<y>nonebot_plugin_apscheduler</y>, <r>禁用定时任务功能</r>"
)


###列表下标0为比赛名称、下标1为比赛时间、下标2为比赛链接

async def ans_today():  # today
    cf_list = await codeforces.get_today_data()
    nc_list = await nowcoder.get_today_data()
    atc_list = await atcoder.get_today_data()

    msg = []
    n = 0
    if len(cf_list) > 0:
        for each in cf_list:
            if n == 0:
                msg.append(cf_title)
            msg.append(to_context(each))

            n += 1
            if n >= 2:
                break
    n = 0
    msg2 = []

    if len(nc_list) > 0:
        # second = '{:.3f}'.format(time.time())
        # second2 = int(float(second)*1000)
        for data in nc_list:
            if n == 0:
                msg2.append(nc_title)
            msg2.append(to_context(data))
            n += 1
            if n >= 2:
                break
    n = 0
    msg3 = []

    if len(atc_list) > 0:
        for each in atc_list:
            if n == 0:
                msg3.append(atc_title)
            msg3.append(to_context(each))

    if len(cf) == 0 and len(nc) == 0 and len(atc) == 0:
        return f'查询出错了，稍后再尝试哦~'
    if len(cf_list) == 0 and len(nc_list) == 0 and len(atc_list) == 0:
        return '今天没有比赛，但也要好好做题哦~'
    return [ '找到今天的比赛如下：', ''.join(msg), ''.join(msg2), ''.join(msg3) ]


async def ans_next():
    cf_list = await codeforces.get_next_data()
    nc_list = await nowcoder.get_next_data()
    atc_list = await atcoder.get_next_data()
    tomorrow = datetime.datetime.now().date() + timedelta(days=1)
    msg = []
    n = 0

    if len(cf_list) > 0:
        for each in cf_list:
            if n == 0:
                msg.append(cf_title)
            msg.append(to_context(each))
            n += 1
            if n >= 2:
                break
    n = 0
    msg2 = []

    if len(nc_list) > 0:
        for data in nc_list:
            if n == 0:
                msg2.append(nc_title)
            msg2.append(to_context(data))
            n += 1
            if n >= 2:
                break

    n = 0
    msg3 = []

    if len(atc_list) > 0:
        for each in atc_list:
            if n == 0:
                msg3.append(atc_title)
            msg3.append(to_context(each))
            n += 1

    if len(cf) == 0 and len(nc) == 0 and len(atc) == 0:
        return f'查询出错了，稍后再尝试哦~'
    if len(cf_list) == 0 and len(nc_list) == 0 and len(atc_list) == 0:
        return '接下来几天没有比赛，但也要好好做题哦~'
    return ['找到接下来的比赛如下：', ''.join(msg), ''.join(msg2), ''.join(msg3)]


today_matcher = on_fullmatch('today', priority=70, block=True)


@today_matcher.handle()
async def _(bot: Bot, event: MessageEvent):
    msg = await ans_today()
    if isinstance(msg, str):
        await today_matcher.finish(msg)
    else:
        await send_forward_msg(bot, event, "今日比赛", bot.self_id, msg)


next_matcher = on_fullmatch('next', priority=70, block=True)


@next_matcher.handle()
async def _(bot: Bot, event: MessageEvent):
    msg = await ans_next()
    if isinstance(msg, str):
        await next_matcher.finish(msg)
    else:
        await send_forward_msg(bot, event, "接下来比赛", bot.self_id, msg)


help = on_fullmatch(('help', '帮助'), block=True, priority=70, rule=to_me())


@help.handle()
async def help_():
    msg = __plugin_meta__.usage.format(botname=cp_broadcast_botname)
    await help.finish(msg)


# 清晨自动播报比赛
async def auto_broadcast():
    await asyncio.sleep(1)
    for id in cp_broadcast_list:
        await asyncio.sleep(2)
        await get_bot().send_group_msg(group_id=id, message='早上好呀！' + await ans_today())


if scheduler:
    scheduler.add_job(auto_broadcast,
                      "cron", hour=cp_broadcast_time['hour'], minute=cp_broadcast_time['minute'], id="auto_broadcast"
                      )


# 凌晨12点自动更新比赛信息
async def update():
    global cf
    global atc
    global nc
    cf.clear()
    atc.clear()
    nc.clear()

    if await get_data_cf():
        logger.info('cf比赛信息更新成功')
    if await get_data_nc():
        logger.info('牛客比赛信息更新成功')
    if await get_data_atc():
        logger.info('atc比赛信息更新成功')


if scheduler:
    scheduler.add_job(
        update, "cron", hour=cp_broadcast_updatetime['hour'], minute=cp_broadcast_updatetime['minute'], id="update"
    )

# 更新数据
update_matcher = on_fullmatch('update', priority=70, block=True)


@update_matcher.handle()
async def reply():
    await update()
    await update_matcher.finish('数据已更新完成')


# cf分数变化提醒、cf上线提醒
async def cfBroadcast():
    """
    Users = {'ratingChange': [
                {'handle': handle, 'remarks' : remarks,'oldRating': Rating.oldRating, 'newRating': Rating.newRating}
                            ],
            'cfOnline': [
                {'handle': handle, 'remarks' : remarks, 'contestId': contestId}
                            ]}
    """
    logger.info('cf监视工作开始')
    messList = await returnChangeInfo()
    logger.info('cf监视工作完成，结果已返回')

    for id in cp_broadcast_cf_list:
        await asyncio.sleep(1)
        output = f"当前时间：{time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())}\n"
        if len(messList['ratingChange']) != 0:
            for mess in messList['ratingChange']:
                output += f"cf用户 {mess['handle']} ({mess['remarks']})分数发生变化，从 {mess['oldRating']} → {mess['newRating']}，变动了{int(mess['newRating'] - int(mess['oldRating']))}分！\n"
            await get_bot().send_group_msg(group_id=id, message=output)
            await asyncio.sleep(1)

        for mess in messList['cfOnline']:
            await get_bot().send_group_msg(group_id=id,
                                           message=f"卷王 {mess['handle']} ({mess['remarks']})又开始上cf做题啦！他在做{mess['contestId']}！\n")
            await asyncio.sleep(1)


if scheduler:
    scheduler.add_job(
        cfBroadcast, "interval", minutes=cp_broadcast_cf_interval, id="cfBroadcast"
    )



import datetime
import time

from json import loads
from html import unescape
from bs4 import BeautifulSoup, NavigableString, ResultSet, Tag
from httpx import AsyncClient, Response
from httpx._types import URLTypes, ProxiesTypes

from nonebot.plugin import on_fullmatch
from fake_useragent import FakeUserAgent
from nonebot.adapters.onebot.v11.message import Message
import asyncio
from typing import Any, Dict, Literal, Optional, TypedDict, List, Union

from .config import ContestType
from .utils import *

headers = {
    'user-agent': FakeUserAgent().random
}

###列表下标0为比赛名称、下标1为比赛时间、下标2为比赛链接
nc: List[ContestType] = []


async def req_get(url: URLTypes, proxies: Optional[ProxiesTypes] = None) -> str:
    """
    生成一个异步的GET请求

    Args:
        url (URLTypes): 对应的URL

    Returns:
        str: URL对应的HTML
    """
    async with AsyncClient(proxies=proxies) as client:
        r: Response = await client.get(url)
    return r.content.decode("utf-8")


async def get_data_nc(url: str = "https://ac.nowcoder.com/acm/contest/vip-index") -> List[ContestType]:
    """
    处理牛客的竞赛列表

    Args:
        content (str): HTML

    Returns:
        list: 竞赛列表
    """

    global nc
    content = await req_get(url)
    soup = BeautifulSoup(content, 'html.parser')
    find_item: Union[Tag, NavigableString, None] = soup.find('div', class_='platform-mod js-current')
    if not isinstance(find_item, Tag):
        return nc
    datatable: ResultSet = find_item.find_all('div', class_='platform-item js-item')

    for contest in datatable:
        cdata: Dict[str, Any] = loads(unescape(contest.get("data-json")))
        id = unescape(contest.get("data-id"))
        cdata["contestId"] = id
        contest_url = f"https://ac.nowcoder.com/acm/contest/{cdata['contestId']}"
        if cdata:
            nc.append(ContestType(cdata['contestName'], int(cdata["contestStartTime"] / 1000),
                                  contest_url, cdata["contestDuration"] / 1000))

    return nc


async def get_today_data() -> List[ContestType]:
    """

    :return: 返回今天的牛客比赛
    """
    global nc
    contest_list: List[ContestType] = []
    if len(nc) == 0:
        await get_data_nc()

    if len(nc) > 0:
        # second = '{:.3f}'.format(time.time())
        # second2 = int(float(second)*1000)

        today = datetime.datetime.now().date()
        for each in nc:
            cur_time = datetime.datetime.strptime(str(each.get_time()), "%Y-%m-%d %H:%M").date()
            if cur_time == today:
                contest_list.append(each)

    return contest_list


async def get_next_data() -> List[ContestType]:
    """

    :return: 返回接下来的比赛
    """
    global nc
    msg2 = ''
    n = 0
    contest_list: List[ContestType] = []
    if len(nc) == 0:
        await get_data_nc()

    if len(nc) > 0:
        # second = '{:.3f}'.format(time.time())
        # second2 = int(float(second)*1000)

        tomorrow = datetime.datetime.now().date() + datetime.timedelta(days=1)
        for each in nc:
            cur_time = datetime.datetime.strptime(str(each.get_time()), "%Y-%m-%d %H:%M").date()
            if cur_time >= tomorrow:
                contest_list.append(each)

    return contest_list


async def ans_nc() -> str:
    """

    :return: 返回牛客比赛的信息
    """
    global nc
    if len(nc) == 0:
        await get_data_nc()
    if len(nc) == 0:
        return f'突然出错了，稍后再试哦~'

    # second = '{:.3f}'.format(time.time())
    # second2 = int(float(second)*1000)
    msg = []
    n = 0
    for each in nc:
        n += 1
        msg.append(to_context(each))
        if n == 3:
            break
    return '\n'.join([f"找到最近的 {n} 场牛客比赛为：\n"] + msg)


nc_matcher = on_fullmatch(('牛客', 'nc'), priority=70, block=True)


@nc_matcher.handle()
async def _():
    msg = await ans_nc()
    await nc_matcher.finish(msg)



# import urllib.request
import time

from httpx import AsyncClient
import datetime
from bs4 import BeautifulSoup
from nonebot.plugin import on_fullmatch
from nonebot.adapters.onebot.v11.message import Message
from nonebot.log import logger
from typing import List
from .config import *
from .utils import *
import asyncio
import re

###列表下标0为比赛名称、下标1为比赛时间、下标2为比赛链接
atc: List[ContestType] = []


async def get_today_data() -> List[ContestType]:
    global atc
    if len(atc) == 0:
        await get_data_atc()

    today = datetime.datetime.now().date()
    contest_list: List[ContestType] = []
    if len(atc) > 0:
        for each in atc:
            cur_time = datetime.datetime.strptime(str(each.get_time()), "%Y-%m-%d %H:%M").date()
            if cur_time == today:
                contest_list.append(each)

    return contest_list


async def get_next_data() -> List[ContestType]:
    global atc
    if len(atc) == 0:
        await get_data_atc()

    tomorrow = datetime.datetime.now().date() + datetime.timedelta(days=1)
    contest_list: List[ContestType] = []
    if len(atc) > 0:
        for each in atc:
            cur_time = datetime.datetime.strptime(str(each.get_time()), "%Y-%m-%d %H:%M").date()
            if cur_time >= tomorrow:
                contest_list.append(each)

    return contest_list


async def get_data_atc() -> bool:  # 以元组形式插入列表中，从左到右分别为比赛名称、比赛时间、比赛链接
    global atc
    url = f'https://atcoder.jp/contests/?lang=en'
    num = 0  # 爬取次数，最多爬三次
    while num < 3:
        try:
            if len(atc) > 0:
                atc.clear()

            async with AsyncClient() as client:
                resp = await client.get(url=url, timeout=10.0)

            soup = \
                BeautifulSoup(resp.text, 'lxml').find_all(name='div', attrs={'id': 'contest-table-upcoming'})[
                    0].find_all(
                    'tbody')[0].find_all('td')
            contest1_name = str(soup[1].contents[5].contents[0])
            ss = str(soup[0].contents[0].contents[0].contents[0]).replace('+0900', '')
            contest1_time1 = str(datetime.datetime.strptime(ss, '%Y-%m-%d %H:%M:%S') - datetime.timedelta(hours=1))
            contest1_time = int(time.mktime(time.strptime(contest1_time1, "%Y-%m-%d %H:%M:%S")))
            url1 = 'https://atcoder.jp' + re.findall(r'<a href="(.+?)">', str(soup[1]))[0]
            contest1_length = re.findall(r'<td class="text-center">(.+?)</td>', str(soup[2]))[0]
            length_list1 = contest1_length.split(':')

            contest2_name = str(soup[5].contents[5].contents[0])
            ss1 = str(soup[4].contents[0].contents[0].contents[0]).replace('+0900', '')
            contest2_time2 = str(datetime.datetime.strptime(ss1, '%Y-%m-%d %H:%M:%S') - datetime.timedelta(hours=1))
            contest2_time = int(time.mktime(time.strptime(contest2_time2, "%Y-%m-%d %H:%M:%S")))
            url2 = 'https://atcoder.jp' + re.findall(r'<a href="(.+?)">', str(soup[5]))[0]
            contest2_length: str = re.findall(r'<td class="text-center">(.+?)</td>', str(soup[2]))[0]
            length_list2 = contest2_length.split(':')

            atc.append(ContestType(contest1_name, contest1_time, url1, int(length_list1[0]) * 3600 + int(length_list1[1]) * 60))
            atc.append(ContestType(contest2_name, contest2_time, url2, int(length_list2[0]) * 3600 + int(length_list2[1]) *60))

            return True
        except Exception as e:
            logger.warning(str(e))
            num += 1
            await asyncio.sleep(2)
    return False


async def ans_atc() -> str:
    global atc
    try:
        if len(atc) == 0:
            await get_data_atc()
        if len(atc) == 0:
            return f'突然出错了，稍后再试哦~'
        msg = []
        for each in atc:
            msg.append(to_context(each))
        return '\n'.join([f"找到最近的 2 场atc比赛为：\n"] + msg)
    except:
        return f'突然出错了，稍后再试哦~'


atc_matcher = on_fullmatch('atc', priority=80, block=True)


@atc_matcher.handle()
async def reply_handle():
    msg = await ans_atc()
    await atc_matcher.finish(msg)



import time

from pydantic import BaseModel, Extra
from typing import List, Dict, TypedDict
from pathlib import Path

from nonebot import get_driver, require

require("nonebot_plugin_localstore")

from nonebot_plugin_localstore import get_data_dir


class Config(BaseModel, extra=Extra.ignore):
    cp_broadcast_path: str = get_data_dir("nonebot_plugin_cp_broadcast")
    cp_broadcast_list: List[str] = []
    cp_broadcast_botname: str = "bot"
    cp_broadcast_time: Dict[str, str] = {"hour": "7", "minute": "20"}
    cp_broadcast_updatetime: Dict[str, str] = {"hour": "0", "minute": "0"}
    cp_broadcast_cf_list: List[str] = []
    cp_broadcast_cf_interval: int = 10


class ContestType:
    def __init__(self, contest_name: str, contest_time: int, contest_url: str, contest_length: int):
        self.contest_name = contest_name
        self.contest_time = contest_time
        self.contest_url = contest_url
        self.contest_length = contest_length

    def get_name(self) -> str:
        return self.contest_name

    def get_time(self) -> str:
        return time.strftime("%Y-%m-%d %H:%M", time.localtime(self.contest_time))

    def get_url(self) -> str:
        return self.contest_url

    def get_length(self) -> str:
        return f"{int(self.contest_length / 60)}"


cf_user_info_baseurl = 'https://codeforces.com/api/user.info?handles='
cf_user_status_baseurl = 'https://codeforces.com/api/user.status?handle={handle}&from=1&count=1'  # 查询交题记录，仅返回一个
cf_user_rating_baseurl = 'https://codeforces.com/api/user.rating?handle={handle}'

cf_title = '◉Codeforces比赛：\n'
nc_title = '◉牛客比赛：\n'
atc_title = '◉AtCoder比赛：\n'

cp_broadcast_config = Config.parse_obj(get_driver().config.dict())
cp_broadcast_path = Path(cp_broadcast_config.cp_broadcast_path)



from typing import List

from nonebot.adapters.onebot.v11 import (
    Bot,
    MessageEvent,
    GroupMessageEvent,
)

from .config import ContestType


def to_context(contest: ContestType) -> str:
    return '\n'.join([
        f'比赛名称：{contest.get_name()}',
        f'比赛时间：{contest.get_time()}',
        f'比赛链接：{contest.get_url()}',
        f'比赛时长：{contest.get_length()}分钟',
        f'',
    ])


async def send_forward_msg(
        bot: Bot,
        event: MessageEvent,
        name: str,
        uin: str,
        msgs: List[str],
):
    def to_json(msg):
        return {"type": "node", "data": {"name": name, "uin": uin, "content": msg}}

    messages = [to_json(msg) for msg in msgs]
    if isinstance(event, GroupMessageEvent):
        await bot.call_api(
            "send_group_forward_msg", group_id=event.group_id, messages=messages
        )
    else:
        await bot.call_api(
            "send_private_forward_msg", user_id=event.user_id, messages=messages
        )



from .config import (cp_broadcast_path, cf_user_info_baseurl,
                     cf_user_status_baseurl, cf_user_rating_baseurl)
from nonebot.log import logger
import sqlite3
import aiosqlite
import json
import datetime
import time
from httpx import AsyncClient
from nonebot.adapters.onebot.v11 import Message, MessageSegment
import asyncio
from nonebot import get_driver

# 激活驱动器
driver = get_driver()

conn: aiosqlite.Connection
cursor: aiosqlite.Cursor


@driver.on_startup
async def _on_startup():
    if not cp_broadcast_path.exists():
        cp_broadcast_path.mkdir(parents=True, exist_ok=True)

    global conn, cursor
    conn = await aiosqlite.connect(cp_broadcast_path / 'data.db')
    cursor = await conn.cursor()
    await cursor.execute("""
        CREATE TABLE IF NOT EXISTS CF_User_info (
            handle TEXT PRIMARY KEY,   
            contribution INTEGER,       
            'rank' TEXT,               
            rating INTEGER,            
            maxRank TEXT,              
            maxRanting INTEGER,        
            lastOnlineTimeSeconds INTEGER, 
            friendOfCount INTEGER,     
            avatar TEXT            
        )
    """
                         )
    # codeforces' username
    # User contribution.
    # User rank, String.
    # 现在的rating分
    # 最高的rank
    # 最高的rating分
    # 上次在线时间
    # The user's friend count
    # 头像链接

    await cursor.execute("""
        CREATE TABLE IF NOT EXISTS CF_User_status (  
            handle TEXT PRIMARY KEY,   
            id INTEGER,                
            contestId INTEGER,         
            programmingLanguage TEXT,  
            passedTestCount INTEGER,   
            timeConsumedMillis INTEGER, 
            memoryConsumedBytes INTEGER 
            
        )
    """
                         )
    # 记录user的submission表
    # codeforces' username
    # one submission's id.
    # 比赛的id
    # 程序所用语言
    # 通过的数据组数
    # Maximum time in milliseconds, consumed by solution for one test.
    # Maximum memory in bytes, consumed by solution for one test.

    await cursor.execute("""
        CREATE TABLE IF NOT EXISTS CF_User_rating (  
            handle TEXT PRIMARY KEY,   
            contestId INTEGER,         
            contestName TEXT,          
            'rank' INTEGER,            
            oldRating INTEGER,         
            newRating INTEGER          
        )
    """
                         )

    # 记录user的rating 变化记录，只记录最后一条
    # codeforces' username
    # contest's id.
    # 比赛的名称
    # 比赛排名
    # 变化前排名
    # 变化后排名

    await cursor.execute("""
        CREATE TABLE IF NOT EXISTS CF_User_remarks (  
            handle TEXT PRIMARY KEY,   
            remarks TEXT,
            broadcast_time INTEGER    
        )
    """
                         )

    # user的id
    # 该user的备注名
    # 更新时间


class CF_UserInfo:
    def __init__(self, handle, contribution, rank, rating, maxRank, maxRating,
                 lastOnlineTimeSeconds, friendOfCount, avatar):
        self.handle = str(handle).lower()
        self.contribution = int(contribution)
        self.rank = str(rank)
        self.rating = int(rating)
        self.maxRank = str(maxRank)
        self.maxRating = int(maxRating)
        self.lastOnlineTimeSeconds = int(lastOnlineTimeSeconds)
        self.friendOfCount = int(friendOfCount)
        self.avatar = str(avatar)

    def returnTuple(self):
        return (self.handle, self.contribution, self.rank, self.rating, self.maxRank, self.maxRating,
                self.lastOnlineTimeSeconds, self.friendOfCount, self.avatar)

    @staticmethod
    async def getByHttp(handle: str):
        cf_user_info_url = cf_user_info_baseurl + handle
        try:
            await asyncio.sleep(0.5)
            async with AsyncClient() as client:
                response = await client.get(cf_user_info_url, timeout=10.0)
            response.raise_for_status()

            data = json.loads(response.text)
        except Exception as e:
            logger.warning(e)
            return None

        if data["status"] == "OK":
            for result in data["result"]:
                Info = CF_UserInfo(
                    handle=result["handle"],
                    contribution=result["contribution"],
                    rank=result["rank"] if "rank" in result else "null",
                    rating=result["rating"] if "rating" in result else 0,
                    maxRank=result["maxRank"] if "maxRank" in result else "null",
                    maxRating=result["maxRating"] if "maxRating" in result else 0,
                    lastOnlineTimeSeconds=result["lastOnlineTimeSeconds"] if "lastOnlineTimeSeconds" in result else 0,
                    friendOfCount=result["friendOfCount"] if "friendOfCount" in result else 0,
                    avatar=result["avatar"] if "avatar" in result else "404"
                )

                return Info

        logger.warning('请求失败')
        return None


class CF_UserStatus:
    def __init__(self, handle, id, contestId, programmingLanguage,
                 passedTestCount, timeConsumedMillis, memoryConsumedBytes):
        self.handle = str(handle).lower()
        self.id = int(id)
        self.contestId = int(contestId)
        self.programmingLanguage = str(programmingLanguage)
        self.passedTestCount = int(passedTestCount)
        self.timeConsumedMillis = int(timeConsumedMillis)
        self.memoryConsumedBytes = int(memoryConsumedBytes)

    def returnTuple(self):
        return (self.handle, self.id, self.contestId, self.programmingLanguage,
                self.passedTestCount, self.timeConsumedMillis, self.memoryConsumedBytes)

    @staticmethod
    async def getByHttp(handle: str):
        cf_user_status_url = cf_user_status_baseurl.format(handle=handle)
        try:
            await asyncio.sleep(0.5)
            async with AsyncClient() as client:
                response = await client.get(cf_user_status_url, timeout=10.0)
            response.raise_for_status()

            data = json.loads(response.text)
        except Exception as e:
            logger.warning(e)
            return None

        if data["status"] != "OK":
            logger.warning('请求失败')
            return None

        if len(data["result"]) == 0:
            return None

        result = data["result"][0]
        Status = CF_UserStatus(
            handle=handle,
            id=result["id"],
            contestId=result["contestId"] if "contestId" in result else 0,
            programmingLanguage=result["programmingLanguage"],
            passedTestCount=result["passedTestCount"],
            timeConsumedMillis=result["timeConsumedMillis"],
            memoryConsumedBytes=result["memoryConsumedBytes"]
        )

        return Status


class CF_UserRating:
    def __init__(self, handle, contestId, contestName, rank, oldRating, newRating):
        self.handle = str(handle).lower()
        self.contestId = int(contestId)
        self.contestName = str(contestName)
        self.rank = str(rank)
        self.oldRating = int(oldRating)
        self.newRating = int(newRating)

    def returnTuple(self):
        return (self.handle, self.contestId, self.contestName,
                self.rank, self.oldRating, self.newRating)

    @staticmethod
    async def getByHttp(handle: str):
        cf_user_rating_url = cf_user_rating_baseurl.format(handle=handle)
        try:
            await asyncio.sleep(0.5)
            async with AsyncClient() as client:
                response = await client.get(cf_user_rating_url, timeout=10.0)
            response.raise_for_status()

            data = json.loads(response.text)
        except Exception as e:
            logger.warning(e)
            return None

        if data["status"] != "OK":
            logger.warning('请求失败')
            return None

        if len(data["result"]) == 0:
            return None

        result = data["result"][-1]

        Rating = CF_UserRating(
            handle=result["handle"],
            contestId=result["contestId"],
            contestName=result["contestName"],
            rank=result["rank"],
            oldRating=result["oldRating"],
            newRating=result["newRating"]
        )

        return Rating


class CF_UserRemarks:
    def __init__(self, handle: str, remarks: str, broadcast_time: int):
        self.handle = handle.lower()
        self.remarks = remarks.lower()
        self.broadcast_time = broadcast_time

    def returnTuple(self):
        return (self.handle, self.remarks, self.broadcast_time)


async def addUser(id: str):
    global conn, cursor
    Info = await CF_UserInfo.getByHttp(id)
    Status = await CF_UserStatus.getByHttp(id)
    Rating = await CF_UserRating.getByHttp(id)
    Remarks = CF_UserRemarks(id, id, 0)

    status = True

    if Info is not None:
        await cursor.execute('INSERT OR REPLACE INTO CF_User_info VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)',
                             Info.returnTuple())
    else:
        status = False

    if Status is not None:
        await cursor.execute('INSERT OR REPLACE INTO CF_User_status VALUES (?, ?, ?, ?, ?, ?, ?)', Status.returnTuple())
    # else:
    #     status = False

    if Rating is not None:
        await cursor.execute('INSERT OR REPLACE INTO CF_User_rating VALUES (?, ?, ?, ?, ?, ?)', Rating.returnTuple())
    # else:
    #     status = False

    await cursor.execute('INSERT OR REPLACE INTO CF_User_remarks VALUES (?, ?, ?)', Remarks.returnTuple())

    await conn.commit()

    return status


async def removeUser(id: str):
    global cursor, conn

    cfid = id.lower()
    await cursor.execute('DELETE FROM CF_User_info WHERE handle = ?', (cfid,))
    await cursor.execute('DELETE FROM CF_User_status WHERE handle = ?', (cfid,))
    await cursor.execute('DELETE FROM CF_User_rating WHERE handle = ?', (cfid,))
    await cursor.execute('DELETE FROM CF_User_remarks WHERE handle = ?', (cfid,))
    await conn.commit()

    if cursor.rowcount == 0:
        return False

    return True


async def returnChangeInfo():
    Users = {'ratingChange': [], 'cfOnline': []}
    global cursor, conn

    await cursor.execute('''
        SELECT CF_User_info.handle, rating, CF_User_status.id, remarks, broadcast_time
        FROM CF_User_info, CF_User_status, CF_User_remarks 
        WHERE CF_User_info.handle=CF_User_status.handle and CF_User_info.handle=CF_User_remarks.handle
    ''')
    RS = await cursor.fetchall()
    for row in RS:
        handle = row[0]
        rating = row[1]
        submission_id = row[2]
        remarks = row[3]
        broadcast_time = int(row[4])

        Info = await CF_UserInfo.getByHttp(handle)
        Status = await CF_UserStatus.getByHttp(handle)
        Rating = await CF_UserRating.getByHttp(handle)

        # -----在这片区域可以写前后变化的操作
        ## 分数变化，以json形式存入
        if Rating is not None and Rating.newRating != rating:
            Users['ratingChange'].append(
                {'handle': handle, 'remarks': remarks, 'oldRating': Rating.oldRating, 'newRating': Rating.newRating})

        ## cf上线提醒，有交题，且两小时内没播报说明在卷
        if (Status is not None and Status.id != submission_id and
                (int(time.time()) - broadcast_time >= 7200)):
            await cursor.execute('update CF_User_remarks set broadcast_time=? where handle=?',
                                 (int(time.time()), handle))
            Users['cfOnline'].append({'handle': handle, 'remarks': remarks, 'contestId': Status.contestId})
        # -----

        if Info is not None:
            await cursor.execute('INSERT OR REPLACE INTO CF_User_info VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)',
                                 Info.returnTuple())

        if Status is not None:
            await cursor.execute('INSERT OR REPLACE INTO CF_User_status VALUES (?, ?, ?, ?, ?, ?, ?)',
                                 Status.returnTuple())

        if Rating is not None:
            await cursor.execute('INSERT OR REPLACE INTO CF_User_rating VALUES (?, ?, ?, ?, ?, ?)',
                                 Rating.returnTuple())

        await conn.commit()

    return Users


async def returnBindList():
    global cursor, conn
    await cursor.execute('SELECT handle FROM CF_User_info')
    data = await cursor.fetchall()
    if data is None:
        return f'当前无监视选手'

    msg = '当前已监视选手如下:\n'

    for curTuple in data:
        handle = str(curTuple[0])
        await cursor.execute('SELECT remarks FROM CF_User_remarks WHERE handle=?', (handle,))
        data = await cursor.fetchone()
        if data is None:
            remarks = "null"
        else:
            remarks = str(data[0])
        msg += f'{handle}({remarks}) \n'
    return msg


async def queryUser(id: str):
    global cf_user_info_baseurl, cursor, conn
    cf_user_info_url = cf_user_info_baseurl + id

    msg = MessageSegment.text('查询失败！请检查该用户是否存在')
    try:
        async with AsyncClient() as client:
            response = await client.get(cf_user_info_url, timeout=10.0)
        response.raise_for_status()

        data = json.loads(response.text)
    except Exception as e:
        logger.warning(e)
        return msg

    if data['status'] == 'OK':
        results = data['result']

        for result in results:
            avatar_url = result['avatar'] if "avatar" in result else "null"
            name = result['handle']
            rank = result['rank'] if 'rank' in result else 'Unrated'
            contest_rating = result['rating'] if 'rating' in result else '0'
            max_rating = result['maxRating'] if 'maxRating' in result else '0'
            contribution = result['contribution'] if 'contribution' in result else '0'
            friend_of = result['friendOfCount'] if 'friendOfCount' in result else '0'

            async with AsyncClient() as client:
                resp = await client.get(avatar_url, timeout=10.0)

            pic = resp.content

            msg = MessageSegment.image(pic)
            msg += Message(
                "\nname: " + name + \
                "\nrank: " + rank + \
                "\nrating: " + str(contest_rating) + \
                "\nmax rating: " + str(max_rating) + \
                "\ncontribution: " + str(contribution) + \
                "\nfriend of: " + str(friend_of) + " users"
            )

            return msg
    else:
        logger.warning('添加用户失败')
        return msg


async def returnRanklist():
    global cursor, conn
    await cursor.execute('SELECT handle, rating FROM CF_User_info ORDER BY rating DESC')
    data = await cursor.fetchall()
    return data


async def modifyRemarks(cf_id: str, cf_remarks: str):
    global cursor, conn
    cfid = cf_id.lower()

    await cursor.execute(
        f"insert or replace into CF_User_remarks(handle, remarks) select handle, '{cf_remarks}' from CF_User_info where handle=?",
        (cfid,))
    await conn.commit()
    if cursor.rowcount == 0:
        return False
    return True
